unit MMMainFrm;
{------------------------------------------------------------------------------}
(*
  Very simple "mail merging" application (text search version)
  It loads a template from TEMPLATE.RVF, and data from CUSTOMERS.TXT.
  Fields are any strings inside {}
  This demo also align fields to the right
*)
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVScroll, RichView, StdCtrls, RVStyle, RVTable, CRVData, RVEdit;

type

  TPerson = class (TCollectionItem)
    public
      Name, Address1, Address2, Zip, City, Country: String;
  end;


  TForm1 = class(TForm)
    rvOutput: TRichView;
    rvsOutput: TRVStyle;
    rvsTemplate: TRVStyle;
    Label1: TLabel;
    Label2: TLabel;
    rve: TRichViewEdit;
    Button1: TButton;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    Persons: TCollection;
    Index: Integer;
    procedure LoadPersons;
    procedure CreateTemplate;
    function GetFieldValueFromDatabase(const FieldName: String): String;
    procedure FillFields(RVData: TCustomRVData; StartItemNo: Integer);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}
{ This procedure loads file CUSTOMERS.TXT into Persons.
  In real applications data can be stored in some database }
procedure TForm1.LoadPersons;
var F: TextFile;
    i, Count: Integer;
    Person: TPerson;
begin
  AssignFile(F, ExtractFilePath(Application.ExeName)+'Persons.txt');
  Reset(F);
  try
    Readln(F, Count);
    for i := 0 to Count-1 do
    begin
      Person := Persons.Add as TPerson;
      Readln(F, Person.Name);
      Readln(F, Person.Address1);
      Readln(F, Person.Address2);
      Readln(F, Person.City);
      Readln(F, Person.Zip);
      Readln(F, Person.Country);
    end;
  finally
    CloseFile(F);
  end;
  Label3.Caption := IntToStr(Persons.Count) + ' personal records in the database';
end;
{------------------------------------------------------------------------------}
{ Creating a sample template. You can edit it }
procedure TForm1.CreateTemplate;
begin
   rve.Clear;
   rve.AddNL('To {NAME}', 0, 0);
   rve.AddNL('{ADDRESS1}', 0, 0);
   rve.AddNL('{ADDRESS2}', 0, 0);
   rve.AddNL('{CITY} {ZIP}', 0, 0);
   rve.AddNL('{COUNTRY}', 0, 0);
   rve.AddNL('Letter Sample', 1, 1);
   rve.AddNL('Hello {NAME}', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('You are fired.', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('With best regards,', 0, 0);
   rve.AddNL('your mailing robot.', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('', 0, 0);
   rve.Format;
end;
{------------------------------------------------------------------------------}
{
  This function returns a current field value by the field name.
}
function TForm1.GetFieldValueFromDatabase(const FieldName: String): String;
var Person: TPerson;
begin
  Person := Persons.Items[Index] as TPerson;
  if FieldName='NAME' then
    Result := Person.Name
  else if FieldName='ADDRESS1' then
    Result := Person.Address1
  else if FieldName='ADDRESS2' then
    Result := Person.Address2
  else if FieldName='CITY' then
    Result := Person.City
  else if FieldName='ZIP' then
    Result := Person.ZIP
  else if FieldName='COUNTRY' then
    Result := Person.Country
  else
    Result := '{error: illegal field code}';
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  Persons := TCollection.Create(TPerson);
  LoadPersons;
  CreateTemplate;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormDestroy(Sender: TObject);
begin
  Persons.Free;
  Persons := nil;
end;
{------------------------------------------------------------------------------}
{ Returns paragraph style having all properties of the BaseParaNo-th style,
  but with the specified alignment }
function GetParaNo(RVStyle: TRVStyle; BaseParaNo: Integer;
  AAlignment: TRVAlignment): Integer;
begin
  Result := RVStyle.ParaStyles.FindStyleWithAlignment(BaseParaNo, AAlignment);
  if Result<0 then begin
    with RVStyle.ParaStyles.Add do begin
      Assign(RVStyle.ParaStyles[BaseParaNo]);
      Alignment := AAlignment;
      Standard := False;
    end;
    Result := RVStyle.ParaStyles.Count-1;
  end;
end;
{------------------------------------------------------------------------------}
(*
  This function iterates through all items in RVData starting from StartItemNo,
  searches for strings inside {}, calls GetFieldValueFromDatabase()
  and replaces text between {} (including {}) with the returned value.
  You can move this function to your application unchanged.
*)
procedure TForm1.FillFields(RVData: TCustomRVData; StartItemNo: Integer);
var i,j,r,c: Integer;
    table: TRVTableItemInfo;
    FieldName, FieldValue: String;
    s: String;
    FieldEnd, ParaNo: Integer;
    Changed: Boolean;
begin
  for i := StartItemNo to RVData.ItemCount-1 do
    if RVData.GetItemStyle(i)=rvsTable then begin
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.Rows.Count-1 do
        for c := 0 to table.Rows[r].Count-1 do
          if table.Cells[r,c]<>nil then
            FillFields(table.Cells[r,c].GetRVData, 0);
      end
    else if RVData.GetItemStyle(i)>=0 then begin
      s := RVData.GetItemTextA(i);
      FieldEnd := 0;
      Changed := False;
      for j := Length(s) downto 1 do begin
        if s[j]='}' then
          FieldEnd := j
        else if (s[j]='{') and (FieldEnd>0) then begin
          FieldName :=  Copy(s, j+1, FieldEnd-j-1);
          FieldValue := GetFieldValueFromDatabase(FieldName);
          Delete(s, j, FieldEnd-j+1);
          Insert(FieldValue, s, j);
          FieldEnd := 0;
          Changed := True;
        end;
      end;
      if Changed then begin
        RVData.SetItemTextA(i, s);
        if RVData.IsParaStart(i) and
           ((i+1=RVData.ItemCount) or (RVData.IsParaStart(i+1))) then
          // if the item containing field is the only item in this paragraph,
          // then applying right alignment to it
          RVData.GetItem(i).ParaNo := GetParaNo(RVData.GetRVStyle, RVData.GetItemPara(i), rvaRight);
      end;
    end;
end;
{------------------------------------------------------------------------------}
{ Generating output }
procedure TForm1.Button1Click(Sender: TObject);
var Stream: TStream;
    ItemCount: Integer;
begin
  Index := 0; // index of the current person 
  ItemCount := 0;
  rvOutput.Clear;
  while Index<Persons.Count do begin
    // copying rve to the end of rvOutput
    Stream := TMemoryStream.Create;
    rve.SaveRVFToStream(Stream, False);
    Stream.Position := 0;
    rvOutput.InsertRVFFromStream(Stream, ItemCount);
    Stream.Free;
    // processing the last added copy of template
    if rvOutput.ItemCount>ItemCount then begin
      // starting from new page
      rvOutput.PageBreaksBeforeItems[ItemCount] := True;
      // replacing field codes
      FillFields(rvOutput.RVData, ItemCount);
    end;
    ItemCount := rvOutput.ItemCount;
    inc(Index);
  end;
  rvOutput.Format;
end;

end.
